@@ -26,8 +26,6 @@ module Codec.Picture.Png( -- * High level functions
 
                         , writePng
                         , encodeDynamicPng
-                        , encodePalettedPng
-                        , encodePalettedPngWithMetadata
                         , writeDynamicPng
                         ) where
 
@@ -38,7 +36,11 @@ import Control.Applicative( (<$>) )
 import Control.Arrow( first )
 import Control.Monad( forM_, foldM_, when, void )
 import Control.Monad.ST( ST, runST )
+
+#if !MIN_VERSION_base(4,11,0)
 import Data.Monoid( (<>) )
+#endif
+
 import Data.Binary( Binary( get) )
 
 import qualified Data.Vector.Storable as V
@@ -54,9 +56,9 @@ import Foreign.Storable ( Storable )
 
 import Codec.Picture.Types
 import Codec.Picture.Metadata
-import Codec.Picture.Png.Type
-import Codec.Picture.Png.Export
-import Codec.Picture.Png.Metadata
+import Codec.Picture.Png.Internal.Type
+import Codec.Picture.Png.Internal.Export
+import Codec.Picture.Png.Internal.Metadata
 import Codec.Picture.InternalHelper
 
 -- | Simple structure used to hold information about Adam7 deinterlacing.
